#! usr/bin/perl -w

use strict;
use warnings;

die unless @ARGV==3;

#######################

#input
my $fileA = shift @ARGV;
my $fileB = shift @ARGV;

# mode == 1 : phpBB forum mode
# mode == 2 : html mode
my $mode = shift; #FIXME

#######################

my $br = "";
my $whitespace = " ";
if ($mode ==2){
    $br = "<br>";
    $whitespace = "&nbsp;";
}

my %varsA;
my @varsA;

my %varsB;
my @varsB;

my $versionA = $fileA;
my $versionB = $fileB;

$versionA =~ s/.*?([.\w]*)_reformed.cfg/$1/;
$versionB =~ s/.*?([.\w]*)_reformed.cfg/$1/;

&getData(\$fileA, \@varsA, \%varsA);
&getData(\$fileB, \@varsB, \%varsB);

#######################

#output
my $title = "config_diff_$versionA-$versionB";
my $output;
$output = "txt/config_diff_$versionA-$versionB.txt" if $mode == 0;
$output = "phpBB/config_diff_$versionA-$versionB.phpBB" if $mode == 1;
$output = "html/config_diff_$versionA-$versionB.html" if $mode == 2;


open FO, ">$output";
my $oldfh = select(FO);

#######################
print &subject("$versionA -> $versionB");

$versionA = "$whitespace/* $versionA */";
$versionB = "$whitespace/* $versionB */";

&redColor(\$versionA);
&redColor(\$versionB);

#######################
print &subtitle("Removed elements");

my $removedNum = 0;
foreach my $var (@varsA)
{
    unless (defined $varsB{$var}) {
        print "$versionA $var$varsA{$var}$br\n";
        $removedNum++;
    };
}
print "// Removed : $removedNum elements$br\n";


#######################
print &subtitle("Added elements");

my $addedNum = 0;
foreach my $var (@varsB)
{
    unless (defined $varsA{$var}) {
        print "$versionB $var$varsB{$var}$br\n";
        $addedNum++;
    };
}
print "// Added : $addedNum elements$br\n";


#######################
print &subtitle("Changed elements");


my $changedNum = 0;
foreach my $var (@varsA)
{
    if (defined $varsA{$var}) {
        next if (!(defined $varsB{$var}) || $varsA{$var} eq $varsB{$var});

        $changedNum++;
        print <<EOD
$versionA $var$varsA{$var}$br
$versionB $var$varsB{$var}$br
$br
EOD
;
    };
}

$removedNum = sprintf "% 3d", $removedNum;
$addedNum = sprintf "% 3d", $addedNum;
$changedNum = sprintf "% 3d", $changedNum;

$removedNum =~ s/ /$whitespace/g;
$addedNum =~ s/ /$whitespace/g;
$changedNum =~ s/ /$whitespace/g;

print "// Removed  : $removedNum elements$br\n";
print "// Added $whitespace $whitespace: $addedNum elements$br\n";
print "// Changed  : $changedNum elements$br\n";

#######################
close FO;
select ($oldfh);

exit;

#######################
sub getData($$$)
{
    my ($file, $varsArray, $valueHash) = @_;
    my $var;
    my $value;

    open F, "<$$file";

    while (<F>){
        if (/([^=]*)( = .*)/) {
            $var = $1;
            $value = $2;
            $value =~ s/.*, default = /default = /;
        } else {
            print "FIXME\n";
        }

        if($mode == 1) {
            $var = "[color=blue]${var}[/color]";
        } elsif($mode == 2) {
            $var = "<font color=blue>${var}</font>";
        }

        push @$varsArray, $var;
        $$valueHash{$var} = $value;
    }

    close F;
}

sub subject($)
{
    my $title = shift;

    if($mode == 1) {
        $title = "[size=150]${title}[/size]";
    } elsif($mode == 2) {
        $title = "<font size=+3>${title}</font>";
    }

    return "// $title";
}

sub subtitle($)
{
    my $title = shift;
    $title = "/* == $title == */";

    if($mode == 1) {
        $title = "[b][color=orange][size=120]${title}[/size][/color][/b]";
    } elsif($mode == 2) {
        $title = "<font size=+2 color=orange>${title}</font>";
    }
    return "$br\n$br\n$title$br\n";
}

sub redColor($$)
{
    my $string = shift;

    if ($mode == 1) {
        $$string = "[color=red]$${string}[/color]";
    } elsif ($mode == 2) {
        $$string = "<font color=red>$${string}</font>";
    }
}

